//--------------------------------------------------------------------------------
//
//							Homework #5 - Splines
//
//--------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include <GL/glui.h>		// OpenGL required includes

//----------------------------------------------------------------
// Program Constant Definitions
//----------------------------------------------------------------
#define MAX_DATA_POINTS 50
#define MSWAIT			30  // 30 ms wait => 33 times per sec 

//----------------------------------
// CallBack function id identifiers
//----------------------------------
#define QUIT_EVENT      0
#define IFILE_EVENT     1
#define READ_EVENT      2
#define START_EVENT     3
#define STOP_EVENT      4

#define SCALE_EVENT     15
#define TRANS_EVENT     16
#define RESET_EVENT     17
#define CVIEW_EVENT     18
#define SHOW_EVENT      19
#define SPLINE_EVENT    20

#define CUBIC_SPLINE    1

#define TRUE			1
#define FALSE			0

//----------------------------------------------------------------
// Global Variables
//----------------------------------------------------------------
int		gDebugFlag = 0;
int		gWinID;
GLUI	*pGWin;   // Pointer to Gui Window
       
int		gcur_win_width, gcur_win_height, // holds size and pos.
        gcur_win_x0, gcur_win_y0;        // of drawing area

int		gLast_x, gLast_y;		// handle mouse rotation
float	gRotX, gRotY;
//----------------------------------------------------------------
// Our good Ol' Live Variables
//----------------------------------------------------------------
float	gScale;					//  Scene Scaling
float	gTransXYZ[3];			// Scene Translation
char	gOutputFileName[sizeof(GLUI_String)];
int		gCameraView;
int		gSplineType;          // 0 = linear, 1 = cubic
int		gShowSpline = 1, gShowPts = 1;
int		gAnimate;

int		gSegCounter;	// used in the animate function to determine  
						// what spline segment we are on !!

float	gTime, gDt;		// used to keep track of animation time

char	gStatus[sizeof(GLUI_String)];  // status message to user regarding input file

//--------------------------------------------------------------------------

float gCamView[3];		// Look (View) Direction as specified by user GUIs
float gCamPos[3];       // Camera Position specified by spline/data file


//--------------------------------------------------------------------------
// Define structures to hold our file data and spline coefficients 
//--------------------------------------------------------------------------
typedef struct {
	float *a;			// spline coefficients a,b,c,d
	float *b;
	float *c;
	float *d;
} SPLINE_COEFFS;

typedef struct {
   int   numPts;		// number of data points read in from input file
   float *x;			// x position
   float *y;			// y position
   float *z;			// z position
   float *t;			// time values

   char filename[sizeof(GLUI_String)];  // hold data file name
   bool dataReadIn;						// was data file read in ??? (flag)
} SPLINE_DATA ;

//------------------------------------------------------------------------------
// Structure that holds GUI ptrs, so we can enable and disable them.
//------------------------------------------------------------------------------
typedef struct {
	GLUI_Panel *plotControlPanel;	// Plot Control Panel
	GLUI_Panel *viewTransPanel;		// View Transformation Panel
	GLUI_Panel *cameraAnimPanel;	// Camera Animation Panel
	GLUI_Panel *splineControlPanel;
    GLUI_Rollout *viewRollOut;

	GLUI_StaticText *statusText;
	GLUI_Button *readBtn;           // read button
} GUI_POINTERS;

GUI_POINTERS gMyGui;

SPLINE_DATA data;		   // variable to hold our file data
SPLINE_COEFFS Xc, Yc, Zc;  // hold spline coefficients in X,Y,Z dimensions
int   nSplineSegments;     // equal to (numDataPoints - 1)

//------------------------------------------------------------------------------
// material properties
//------------------------------------------------------------------------------
typedef struct{
	GLfloat ambient[4];    
	GLfloat diffuse[4];   
	GLfloat specular[4];
	GLfloat emission[4];
    GLfloat shine;    
} MATERIAL_TYPE;

MATERIAL_TYPE mat;

//------------------------------------------------------------------------------
// Drawing color definintions
//------------------------------------------------------------------------------
#define WHITE 6
#define BLACK 7                    // Background color
#define RED   0
#define BLUE  4
const float Colors[8][3] = 
{
    { 1., 0., 0. },		// red     0
    { 1., 1., 0. },		// yellow  1
    { 0., 1., 0. },		// green   2
    { 0., 1., 1. },		// cyan    3
    { 0., 0., 1. },		// blue    4
    { 1., 0., 1. },		// magenta 5
    { 1., 1., 1. },		// white   6
    { 0., 0., 0. },		// black   7
};


//-------------------------------------------------------------------------------
// Buncha Function Prototypes
//-------------------------------------------------------------------------------
void InitGraphics(void);
void RenderScene(void);
void CreateControls(void);
void myMouseMotion(int x, int y);
void InitProgramParams(void);
bool ReadDataFile(char *name);
void ResetView(void);
void ComputeSplineCoeffs(void);
void myTimerFunc( int );
void SetTimer(void);
void DisableAllPanels(void);
void EnableAllPanels(void);
void DrawMyScene(void);
void free_all_memory(void);
void CallBackFunc( int );
void myKeyboardFunc( unsigned char , int, int );  // keyboard function.

//---------------------------------------------------------------------------------
// called to compute spline coefficients
//---------------------------------------------------------------------------------
void ComputeSplineCoeffs(void) {
    
	
	if (gSplineType == CUBIC_SPLINE) {
		// compute cubic spline coefficients here




	}else {
		// compute linear spline coefficents.
		for (int i=0; i < data.numPts; i++) {
			Xc.a[i] = data.x[i];                   // Spline coeffs for x data points
			Xc.b[i] = (data.x[i+1] - data.x[i]);
			Xc.c[i] = 0.0; Xc.d[i] = 0.0;

			Yc.a[i] = data.y[i];                   // Spline coeffs for y data points
			Yc.b[i] = (data.y[i+1] - data.y[i]);
			Yc.c[i] = 0.0; Yc.d[i] = 0.0;

			Zc.a[i] = data.z[i];                   // Spline coeffs for z data points
			Zc.b[i] = (data.z[i+1] - data.z[i]);
			Zc.c[i] = 0.0; Zc.d[i] = 0.0;

		}
	}
	nSplineSegments = data.numPts - 1;  // save the number of spline segments

}

//------------------------------------------------------------------------------------
// This function is called by event handler to read data specified by input "name"
//------------------------------------------------------------------------------------
bool ReadDataFile(char *name){

	FILE *fp = fopen(name,"r");

	if (fp) {
		
		fscanf(fp, "%d", &(data.numPts));
		fprintf(stderr, "READFILE: num points = %d \n", data.numPts);
		for (int i=0; i < data.numPts; i++) {
			fscanf(fp,"%f %f %f %f", &data.t[i], &data.x[i],&data.y[i],&data.z[i]);
			fprintf(stderr, "time = %f \tx = %f \ty = %f \tz = %f\n", data.t[i], data.x[i],data.y[i],data.z[i]);
		}
		data.dataReadIn = true;
        fclose(fp);	
	}else{
        data.dataReadIn = false;
	}

	return(data.dataReadIn);
}

//------------------------------------------------------------------------------
//							Typical Main Funtion
//------------------------------------------------------------------------------
int main( int argc, char **argv )
{
   
    glutInit( &argc, argv);		// Initalize OpenGL Toolkit 
    InitGraphics();				// Function to initalize graphics  
    
    //--------------------------------------------------------------------------
	// Note: since we are using a combination window (GUI and drawing area)
	//       display and mouse function registered here. The other functions
	//       are registered CreateControls()
	//--------------------------------------------------------------------------
    glutDisplayFunc( RenderScene );
    glutMotionFunc( myMouseMotion );

    CreateControls();

    glutMainLoop();
    
    return 0;
}

//-----------------------------------------------------------------------------
// compute (x,y,z) at the ith spline segment, given 0<=u<=1
//-----------------------------------------------------------------------------
void GetXYZ(int i, float *x, float *y, float *z, float u){

	*x = Xc.a[i] + Xc.b[i]*u + Xc.c[i]*u*u + Xc.d[i]*u*u*u;
	*y = Yc.a[i] + Yc.b[i]*u + Yc.c[i]*u*u + Yc.d[i]*u*u*u;
	*z = Zc.a[i] + Zc.b[i]*u + Zc.c[i]*u*u + Zc.d[i]*u*u*u;
}
//-----------------------------------------------------------------------------
// Draws data points and spline curve in our scene.
//-----------------------------------------------------------------------------
void PlotDataPoints(void){
	
	float x,y,z;
    float u;
	int i;

	if (gShowPts) {
		glColor3f(Colors[WHITE][0], Colors[WHITE][1], Colors[WHITE][2]);
		for (i=0; i < data.numPts; i++) {
			glPushMatrix();
			glTranslatef(data.x[i], data.y[i], data.z[i]);
			glutWireSphere(0.03,10,10);
			glPopMatrix();
		}
    }

	if (gShowSpline) {
        glColor3f(Colors[BLUE][0], Colors[BLUE][1], Colors[BLUE][2]);
		int num_sub_intervals = 30;
		float du = 1.0/num_sub_intervals;


		glBegin(GL_LINE_STRIP);
		// moveto(x[0], y[0], z[0]);
        glVertex3f(data.x[0], data.y[0], data.z[0]);
		for (i=0; i<nSplineSegments; i++){
            for (u=0; u <= 1.0; u+=du) {       // bug in olfe book
                GetXYZ(i, &x, &y, &z, u);
				// lineto(x,y,z)
				glVertex3f(x,y,z);
			}
		}
        // connect to the end point
		glVertex3f(data.x[data.numPts-1], data.y[data.numPts-1], data.z[data.numPts-1]);
		glEnd();
	}
}
//-----------------------------------------------------------------------------
//					Display Function 
//-----------------------------------------------------------------------------
void RenderScene(void) 
{
    int v, xl, yb;
    
	//---------------------------------------------------------------------
	//  First tell glut to render in drawing window  and not GUI window
	//---------------------------------------------------------------------
	glutSetWindow( gWinID );

    // Clear Screen
	glDrawBuffer( GL_BACK );


	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    //---------------------------------------------------------------------
	// Set Viewport Size: 
	//
	//  Note the main window real-estate is shared by GUI and a drawing
	//  area. Set viewport to be square and to update drawing area only.
	//
	//---------------------------------------------------------------------

    v  = (gcur_win_width < gcur_win_height ? gcur_win_width : gcur_win_height);
    xl = (gcur_win_width - v) / 2;      yb = (gcur_win_height - v) / 2;
    glViewport(xl, yb, v, v);
    
    if( gDebugFlag ) fprintf( stderr, "Display %d %d %d %d\n", xl, yb,  gcur_win_width, gcur_win_height);
    
    // Set view volume
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 1.0, 0.1, 1000.0);
    
    // Set camera to place objects in view volume defined above
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();


    //---------------------------------------------------------------------------
	//						Draw Our Scene - The magic happens here
    //---------------------------------------------------------------------------
	if (gCameraView) {

		gluLookAt(gCamPos[0], gCamPos[1], gCamPos[2],      0,0,0,     0,1,0 );
        DrawMyScene();

	}else{
		
		gluLookAt(gCamView[0], gCamView[1], gCamView[2],   0,0,0,     0,1,0 );

		/* translate the object:									*/
		/* note the minus sign on the z value						*/
		/* this is to make the appearance of the glui z translate	*/
		/* widget more intuitively match the translate behavior		*/
		glTranslatef(gTransXYZ[0], gTransXYZ[1], -gTransXYZ[2]);
		
		// Rotate
		glRotatef(gRotX, 1.0f, 0.0f, 0.0f);
		glRotatef(gRotY, 0.0f, 1.0f, 0.0f);
		
		// scale the scene:						
		glScalef( gScale, gScale, gScale );
		

       if (data.dataReadIn) {
		glDisable( GL_LIGHTING );
			PlotDataPoints();
			glColor3f(Colors[RED][0], Colors[RED][1], Colors[RED][2]);
			glPushMatrix();
				glTranslatef(gCamPos[0], gCamPos[1], gCamPos[2]); // draw camera 
				glutWireSphere(0.1,10,10);
			glPopMatrix(); 
		 glEnable( GL_LIGHTING );
	   }


	   glColor3f(Colors[WHITE][0], Colors[WHITE][1], Colors[WHITE][2]);
       //glutSolidCube(1);

	   DrawMyScene();
	}
	
	glutSwapBuffers();


}

//=====================================================================================
// OpenGL calls this function only once to initalize drawing properties.
//=====================================================================================
void InitGraphics(void) 
{
    if( gDebugFlag ) fprintf( stderr, "InitGraphics\n" );
    glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );

    // create the main window 			
    gWinID = glutCreateWindow( "" );
    glutFullScreen();
 
    // set the drawing window clear color to black
    glClearColor(Colors[BLACK][0], Colors[BLACK][1], Colors[BLACK][2], 1.0f);
    
	// Lights On !!
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

	// Initalize program parameters to some default .. always a good idea
	InitProgramParams();

}


//=====================================================================================
// OpenGL calls this function just after the window is created and before its visible,
// and any time the window is resized by the user.
//=====================================================================================
void ChangeSize( int width, int height )
{
    if( gDebugFlag ) fprintf( stderr, "ChangeSize: w = %d, h = %d ", width, height );  
    
	//---------------------------------------------------------------------
    // deterimine window size and position .... (as required by GLUI)
    // we will set viewport in Display function
	//---------------------------------------------------------------------
    GLUI_Master.get_viewport_area(&gcur_win_x0, &gcur_win_y0, &gcur_win_width, &gcur_win_height);
    if( gDebugFlag ) fprintf( stderr, "%d %d %d %d\n", 
	                  gcur_win_x0, gcur_win_y0,  gcur_win_width, gcur_win_height);    
    
    // Don't need to do anything ...... all is taken care of in Display function
    glutPostRedisplay();
}

void myMouseFunc( int button, int state, int mouseX, int mouseY) {
    if( gDebugFlag ) fprintf( stderr, "myMouseFunc: \n" );  
	// Handle mouse rotation
	if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN ) {
		gLast_x = mouseX; gLast_y = mouseY;
	}
    
}

void myKeyboardFunc( unsigned char key, int x, int y) {
    switch (key) {
	case 's':
		//gShowSpline = !gShowSpline; 
		//CallBackFunc( SHOW_EVENT);
		break;
	case 'p':

		break;
    case 'q' :
		GLUI_Master.close_all();    // shut down glui window before exiting
		free_all_memory();
		exit(0);
		break;
    }
}

void myMouseMotion( int x, int y) {

	gRotX += float(y - gLast_y);
	gRotY += float(x - gLast_x);
	gLast_x = x;
	gLast_y = y;

	glutPostRedisplay();
}



void CallBackFunc( int id) {
	
	if( gDebugFlag ) fprintf(stderr, "In CallBackFunc .... %d \n", id);
		
	switch (id) {
	case IFILE_EVENT: // user changed input filename
		gMyGui.readBtn->enable();       // enable read and plot button
		data.dataReadIn = false;
		DisableAllPanels();
		sprintf(gStatus,"Please enter a filename <push read button>");
	    gMyGui.statusText->set_text(gStatus); 
		glutPostRedisplay();
		break;

	case READ_EVENT: // read button pushed
		if (ReadDataFile(data.filename)) {    // data read in correctly
			gMyGui.readBtn->disable();   // disable read button
			ComputeSplineCoeffs();
			// initalize camera position to first data point!!
		    gCamPos[0] = data.x[0]; gCamPos[1] = data.y[0];gCamPos[2] = data.z[0];
			// finally enable all our processing controls
			EnableAllPanels();
			sprintf(gStatus,"Data Successfully Read");
	        gMyGui.statusText->set_text(gStatus); 
		} else {
			// could not open file ..... notify user
			sprintf(gStatus,"ERROR: Data File Not Found !!!");
	        gMyGui.statusText->set_text(gStatus); 
		}
		break;
	case RESET_EVENT: // 
		// reset user view
		ResetView();
		// reset simulation parameters
        gCamPos[0] = data.x[0]; gCamPos[1] = data.y[0];gCamPos[2] = data.z[0];
		gTime = 0.0;
		gSegCounter   = 0; 

		glutPostRedisplay();
		break;
	case SHOW_EVENT: // User wants to show point or curve
		if (gCameraView) {  // first make sure we are not in camera view
			gShowSpline = 0; 
			gShowPts = 0;
        }
		glutPostRedisplay();
		break;
	case CVIEW_EVENT: // User wants to use camera view

		// probably not a good idea to plot points and spline curve
		if (gCameraView){
			gShowSpline = 0; gShowPts = 0;
			gMyGui.plotControlPanel->disable();
			gMyGui.viewRollOut->disable();
		}else{
			// not using camera view reset everything
			gShowSpline = 1; gShowPts = 1;
			gMyGui.plotControlPanel->enable();
            gMyGui.viewRollOut->enable();
		}
		glutPostRedisplay();

		break;
	
	case START_EVENT: //  handles start event 
		gAnimate = 1;

		SetTimer();   // fire off timer
		break;
	case STOP_EVENT: //   handles stop event
		gAnimate = 0;
		break;

	case SPLINE_EVENT: // user changed spline type to use
		// Recompute Spline coefficients
        ComputeSplineCoeffs();
		break;

	case QUIT_EVENT:
		GLUI_Master.close_all();
		free_all_memory();
		exit(0);
		break;	

	}	

    pGWin->sync_live();

}

void DisableAllPanels(void) {
	gMyGui.cameraAnimPanel->disable();
	gMyGui.plotControlPanel->disable();
	gMyGui.viewTransPanel->disable();
	gMyGui.splineControlPanel->disable();
	gMyGui.viewRollOut->disable();
}
void EnableAllPanels(void) {
	gMyGui.cameraAnimPanel->enable();
	gMyGui.plotControlPanel->enable();
	gMyGui.viewTransPanel->enable();
	gMyGui.splineControlPanel->enable();
	gMyGui.viewRollOut->enable();
}

void CreateControls(void){
    
    // Register main callbacks with glui as required (see glui manual)
    GLUI_Master.set_glutReshapeFunc( ChangeSize );
    GLUI_Master.set_glutKeyboardFunc( myKeyboardFunc );
    GLUI_Master.set_glutMouseFunc( myMouseFunc );
    GLUI_Master.set_glutIdleFunc(NULL);


    // Create glui window
    pGWin = GLUI_Master.create_glui_subwindow( gWinID, GLUI_SUBWINDOW_RIGHT);
    // Tell glui about main drawing window
    pGWin->set_main_gfx_window( gWinID );
    
    //---------------------------------------------------------------------------------------
    // Add controls below
    //---------------------------------------------------------------------------------------

	//------------------------------------------------------
	// create edit box for filename:
	//   callback is called any time edit box is exited
	//------------------------------------------------------
    pGWin->add_statictext(" "); // create blank line
	sprintf(gStatus,"Please enter a filename <push read button>");
	gMyGui.statusText = pGWin->add_statictext(gStatus); 

	GLUI_EditText *p_edit;

    p_edit = pGWin ->add_edittext("Input File:", GLUI_EDITTEXT_TEXT, data.filename,
		                         IFILE_EVENT, (GLUI_Update_CB) CallBackFunc);
	p_edit->set_w(200);
    pGWin->add_statictext(" ");
	gMyGui.readBtn = pGWin->add_button("Read Data File and Plot",  READ_EVENT, (GLUI_Update_CB) CallBackFunc);
	pGWin->add_statictext(" ");

    GLUI_Panel *panel;

	panel = pGWin->add_panel( "Controls" );
	gMyGui.plotControlPanel = pGWin->add_panel_to_panel(panel, "Plot Controls");
    pGWin->add_checkbox_to_panel(gMyGui.plotControlPanel,"Show Points", &gShowPts, SHOW_EVENT, (GLUI_Update_CB) CallBackFunc);
    pGWin->add_checkbox_to_panel(gMyGui.plotControlPanel,"Show Spline", &gShowSpline, SHOW_EVENT, (GLUI_Update_CB) CallBackFunc);

	pGWin->add_column_to_panel(panel, FALSE);
	gMyGui.splineControlPanel = pGWin->add_panel_to_panel(panel, "Spline Controls");
    
	GLUI_RadioGroup *radio;
    radio = pGWin->add_radiogroup_to_panel(gMyGui.splineControlPanel, &gSplineType, SPLINE_EVENT, (GLUI_Update_CB) CallBackFunc);
    pGWin->add_radiobutton_to_group(radio, "Linear Spline");
    pGWin->add_radiobutton_to_group(radio, "Cubic Spline");

	pGWin->add_separator();  

	//--------------------------------------------------------------------
	// Viewing Controls GUI
	//--------------------------------------------------------------------
    
	
	// create a panel for our controls
    gMyGui.viewTransPanel = pGWin->add_panel( "View Transformation" );
    GLUI_Translation *scale, *trans;


	panel = pGWin->add_panel_to_panel(gMyGui.viewTransPanel, "");

	// create a roll out within the panel "View Transformation"
	gMyGui.viewRollOut = pGWin->add_rollout_to_panel(panel, "View Control", false);

	scale = pGWin->add_translation_to_panel(gMyGui.viewRollOut,  "Scale",  GLUI_TRANSLATION_Y , &gScale, SCALE_EVENT, (GLUI_Update_CB) CallBackFunc);
	scale->set_speed( 0.1f );
	
	pGWin->add_column_to_panel(gMyGui.viewRollOut, TRUE );
	trans = pGWin->add_translation_to_panel(gMyGui.viewRollOut, "Trans XY", GLUI_TRANSLATION_XY, &gTransXYZ[0], TRANS_EVENT, (GLUI_Update_CB) CallBackFunc );
	trans->set_speed( 1.0f);
	
	pGWin->add_column_to_panel(gMyGui.viewRollOut,TRUE);
	trans = pGWin->add_translation_to_panel(gMyGui.viewRollOut,  "Trans Z",  GLUI_TRANSLATION_Z , &gTransXYZ[2], TRANS_EVENT, (GLUI_Update_CB) CallBackFunc );
	trans->set_speed( 1.0f);

	pGWin->add_button_to_panel(gMyGui.viewTransPanel, "Reset",  RESET_EVENT, (GLUI_Update_CB) CallBackFunc);

	pGWin->add_separator();  
	//--------------------------------------------------------------------
	// Camera Animation selection GUI 
	//--------------------------------------------------------------------
    // create a panel for our controls
    gMyGui.cameraAnimPanel = pGWin->add_panel( "Camera Animation" );
  	pGWin->add_statictext_to_panel(gMyGui.cameraAnimPanel,"   ");
    pGWin->add_checkbox_to_panel(gMyGui.cameraAnimPanel,"Camera View", &gCameraView, CVIEW_EVENT, (GLUI_Update_CB) CallBackFunc);
	panel = pGWin->add_panel_to_panel(gMyGui.cameraAnimPanel, "",GLUI_PANEL_NONE);
	pGWin->add_button_to_panel(panel,"Start Animation",  START_EVENT, (GLUI_Update_CB) CallBackFunc);
	pGWin->add_column_to_panel(panel,FALSE );
	pGWin->add_button_to_panel(panel,"Stop Animation",  STOP_EVENT, (GLUI_Update_CB) CallBackFunc);


	pGWin->add_separator();  
    pGWin->add_statictext(" ");
 
    pGWin->add_button("QUIT",  QUIT_EVENT, (GLUI_Update_CB) CallBackFunc);

	//--------------------------------------------------------
	// At this point the data file has NOT been read in yet
	// so disable all processing controls, till we have data
	// to work with
	//--------------------------------------------------------
    DisableAllPanels();
}

void ResetView(void) {
    gCamView[0]	= 9.0;  // x - component 350.0
    gCamView[1] = 8.0;  // y - component 400.0
    gCamView[2]	= 8.0;  // z = component 350.0
	
	gLast_x = gLast_y = 0;   // handle mouse rotation
    gRotX   = gRotY = 0.0f;
	gScale = 1.0;
    gTransXYZ[0] = gTransXYZ[1] = gTransXYZ[2] = 0.0f;
}



void InitProgramParams(void){

    ResetView();

	data.t = (float *)calloc(MAX_DATA_POINTS, sizeof(float));
	data.x = (float *)calloc(3*MAX_DATA_POINTS, sizeof(float));
	data.y = (data.x + MAX_DATA_POINTS); data.z = (data.x + 2*MAX_DATA_POINTS);
 

	Xc.a = (float *)calloc(4*MAX_DATA_POINTS, sizeof(float));
    Yc.a = (float *)calloc(4*MAX_DATA_POINTS, sizeof(float));
	Zc.a = (float *)calloc(4*MAX_DATA_POINTS, sizeof(float));


	Xc.b = (Xc.a + MAX_DATA_POINTS); Xc.c = (Xc.a + 2*MAX_DATA_POINTS); Xc.d = (Xc.a + 3*MAX_DATA_POINTS);
    Yc.b = (Yc.a + MAX_DATA_POINTS); Yc.c = (Yc.a + 2*MAX_DATA_POINTS); Yc.d = (Yc.a + 3*MAX_DATA_POINTS);
	Zc.b = (Zc.a + MAX_DATA_POINTS); Zc.c = (Zc.a + 2*MAX_DATA_POINTS); Zc.d = (Zc.a + 3*MAX_DATA_POINTS);

    gSplineType = 0;
	sprintf(data.filename, "./spline_data.dat");
	data.dataReadIn = false;
	sprintf(gOutputFileName, "\0");
    gCameraView = 0;
	gAnimate = 0;
	gTime = 0.0; gDt = float(MSWAIT/1000.0);
}



void SetTimer(void) {
	glutTimerFunc(MSWAIT, myTimerFunc, 0);
}

void myTimerFunc( int value ) {

	//-------------------------------------------------------
	// Compute position of camera and store in gCamPos
	//-------------------------------------------------------

	// first determine what segment we are on and value for u
    float t1, t2, u;

    if (gTime > data.t[gSegCounter + 1]) gSegCounter++; // increment to next segment
	t1 = data.t[gSegCounter]; t2 = data.t[gSegCounter + 1];
	u = (gTime - t1) / (t2 - t1);

    GetXYZ(gSegCounter, &gCamPos[0], &gCamPos[1], &gCamPos[2], u);

	RenderScene();
    
	if (gTime < data.t[data.numPts-1]) {  
	      gTime += gDt;	// time starts at zero
    }else{
		gAnimate = 0;  // end animation reset position to begining
        gCamPos[0] = data.x[0]; gCamPos[1] = data.y[0];gCamPos[2] = data.z[0];
		// reset time
		gTime = 0.0;
		// reset nSegmentCounter
		gSegCounter   = 0; 
		glutPostRedisplay();
	}

	if (gAnimate) SetTimer();


}


//------------------------------------------------------------------------------------------
// No need to look at the code below .... It just draws the scene for our camera animation.
// You dont need to muck with this part of the code ..... Why are you still reading this
// go away .... go on !
//
//						These functions draws our scene.
//------------------------------------------------------------------------------------------
extern "C" {
#include "glm.h"
}

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glut32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glui32.lib")
#pragma comment(lib, "glm.lib")


GLMmodel* pmodel_ball = NULL;
GLMmodel* pmodel_vase = NULL;


//macro to set material property alpha = 1
#define set_prop(a,r,g,b) a[0]=r;a[1]=g;a[2]=b;a[3]=1.0   


void free_all_memory(void) {
    if (data.t) free(data.t);
	if (data.x) free(data.x);
    if (Xc.a)   free(Xc.a);
	if (Yc.a)   free(Yc.a);
	if (Zc.a)   free(Zc.a);
	if (pmodel_ball) glmDelete(pmodel_ball);
	if (pmodel_vase) glmDelete(pmodel_vase);
}

// function to actually set the material properties

void Set_All_Properties(void) {
	glMaterialfv(GL_FRONT, GL_AMBIENT,   mat.ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat.diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR,  mat.specular);
	glMaterialfv(GL_FRONT, GL_EMISSION,  mat.emission);
	glMaterialfv(GL_FRONT, GL_SHININESS, &mat.shine);
}

void tableLeg(double thick, double len)
{
	glPushMatrix();
		glTranslated(0, len/2, 0);
		glScaled(thick, len, thick);
		glutSolidCube(1.0);
	glPopMatrix();
}
void table(double topWid, double topThick, double legThick, double legLen)
{
	// Define table material
    set_prop(mat.ambient,  0.17f, 0.01f, 0.01f);  // ruby
	set_prop(mat.diffuse,  0.61f, 0.04f, 0.04f,);
	set_prop(mat.specular, 0.73f, 0.63f, 0.63f);
	set_prop(mat.emission, 0.0,   0.0,   0.0);
    mat.shine = 77.0f;
    Set_All_Properties();

	glPushMatrix();
		glTranslated(0, legLen, 0);
		glScaled(topWid, topThick, topWid);
		glutSolidCube(1.0);
	glPopMatrix();

	double dist = 0.95 * topWid/2.0 - legThick/2.0;

    set_prop(mat.ambient,  0.2f,  0.2f, 0.2f);
	set_prop(mat.diffuse,  0.1f,  0.1f, 0.1f,);
	set_prop(mat.specular, 0.4f,  0.4f, 0.4f);
	set_prop(mat.emission, 0.0,   0.0,   0.0);
    mat.shine = 2.0f;
    Set_All_Properties();

    glPushMatrix();
		glTranslated(dist, 0, dist);
		tableLeg(legThick, legLen);
		glTranslated(0, 0, -2*dist);
		tableLeg(legThick, legLen);
		glTranslated(-2*dist, 0, 2*dist);
		tableLeg(legThick, legLen);
		glTranslated(0, 0, -2*dist);
		tableLeg(legThick, legLen);
	glPopMatrix();
}

void jackPart(void)
{
	glPushMatrix();
	glScaled(0.2, 0.2, 1.0);
	glutSolidSphere(1, 30, 30);  //30
	glPopMatrix();

	glPushMatrix();
	glTranslated(0, 0, 1.2);
	glutSolidSphere(0.2, 15, 15);
	glTranslated(0, 0, -2.4);
	glutSolidSphere(0.2, 15, 15);
	glPopMatrix();
}

void jack(void)
{
	glPushMatrix();
	jackPart();
    glRotated(90.0, 0, 1, 0);
	jackPart();
	glRotated(90.0, 1, 0, 0);
	jackPart();
	glPopMatrix();
}



void DrawMyScene(void) {

	//****************** Set Lights ****************************
	GLfloat lightIntensity[] = {1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat light_position[] = {1.0f, 1.0f, 1.0f, 0.0f};
	glLightfv(GL_LIGHT0, GL_POSITION,	light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,	lightIntensity);
 	glLightfv(GL_LIGHT0, GL_SPECULAR,	lightIntensity);   

	glPushMatrix();

		glTranslated(-0.3, 0.0, -0.3); 

		glScaled(2.5, 2.5, 2.5);
	// ***************** Draw Jack ****************************
		set_prop(mat.ambient,  0.33f, 0.22f, 0.03f);   // brass
		set_prop(mat.diffuse,  0.78f, 0.57f, 0.11f,);
		set_prop(mat.specular, 0.99f, 0.94f, 0.81f);
		set_prop(mat.emission, 0.0,   0.0,   0.0);
		mat.shine = 28.0f;
        Set_All_Properties();
		glPushMatrix();
		   glTranslated(-0.1, 0.4, 0.2); //0.4, 0.4, 0.6
		   glRotated(45, 0, 0, 1); 	
		   glScaled(0.08, 0.08, 0.08);
		   jack();
		glPopMatrix();

	//****************** Draw Tea Pot *************************
		set_prop(mat.ambient,  0.02f, 0.17f, 0.02f);    // emerald
		set_prop(mat.diffuse,  0.08f, 0.61f, 0.08f,);
		set_prop(mat.specular, 0.63f, 0.73f, 0.63f);
		set_prop(mat.emission, 0.0,   0.0,   0.0);
		mat.shine = 77.0f;
		Set_All_Properties();

		glPushMatrix();
		   glTranslated(-0.2, 0.37, -0.2);
		   glRotated(30, 0, 1, 0);
		   glutSolidTeapot(0.08);
		glPopMatrix();

	//****************** Draw Ball *************************
	float y = 0.0;

	// Alright I cheated !!!
	y = 0.5*(1.1+cos(2*3.14156*gTime))*exp(-0.4*gTime);
	glPushMatrix();
	glTranslated(0.5, y, 0.0);
	if (!pmodel_ball) {
        pmodel_ball = glmReadOBJ("data/soccerball.obj");
        if (!pmodel_ball) exit(0);
        glmUnitize(pmodel_ball);
        glmFacetNormals(pmodel_ball);
        glmVertexNormals(pmodel_ball, 90.0);
    }
	glScaled(0.08, 0.08, 0.08);
	glmDraw(pmodel_ball, GLM_SMOOTH | GLM_MATERIAL);
    glPopMatrix();

	//****************** Draw Vase *************************
	glPushMatrix();
	glTranslated(0.1, 0.5, 0.0);
	if (!pmodel_vase) {
        pmodel_vase = glmReadOBJ("data/vase.obj");
        if (!pmodel_vase) exit(0);
        glmUnitize(pmodel_vase);
        glmFacetNormals(pmodel_vase);
        glmVertexNormals(pmodel_vase, 90.0);
    }
	glScaled(0.2, 0.2, 0.2);
	glmDraw(pmodel_vase, GLM_SMOOTH | GLM_MATERIAL);
    glPopMatrix();

    //****************** Draw Table ***************************
		glPushMatrix();
		   glTranslated(0.0, 0.0, 0.0);   // table center
		   table(0.6, 0.02, 0.02, 0.3); //0.6, 0.02, 0.02, 0.3
		glPopMatrix();

	glPopMatrix();
}